package controller;

import main.*;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observable;

import javax.persistence.metamodel.ManagedType;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import main.Main;

/**
 * Implementierung des Client-Interfaces
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ClientInterfaceImpl extends UnicastRemoteObject implements
		ClientInterface {

	private static final long serialVersionUID = 1L;

	/**
	 * Konstruktor
	 * 
	 * @throws RemoteException
	 */
	public ClientInterfaceImpl() throws RemoteException {
		super();
	}

	/**
	 * Liste von Objekten vom Server empfangen und anwendungsintern an alle
	 * Views weiterreichen
	 */
	public void update(Class t, List<Object> o) throws RemoteException {
		/**
		 * Zeitstempel ausgeben
		 */
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		String time = sdf.format(new Date());
		System.out.println("Daten empfangen: " + time);

		/**
		 * empfangenes Objekt intern weiterreichen
		 */
		Main.controller.delegateObject(t, o);
	}

	/**
	 * Server wurde beendet, entsprechend Clients auch beenden
	 * 
	 */
	@Override
	public void serverTerminated() throws RemoteException {
		/*
		 * Beendigung des Clients soll asynchron zur Beendigung des Servers
		 * laufen
		 */
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane
						.showMessageDialog(
								null,
								"Der Server wurde beendet, der Client kann entsprechend auch nicht mehr weiterlaufen!",
								"Fehler", JOptionPane.ERROR_MESSAGE);
				System.err
						.println("Server wurde beendet, entsprechend wird der Client auch beendet!");
				System.exit(1);
			}
		});
	}
}