package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import main.CustomItemListener;
import main.ResultContent;
import main.TextLengthDocument;
import main.UserRole;

import controller.Controller;
import databaseModel.Ansprechpartner;
import databaseModel.Unternehmen;

/**
 * Klasse zum Neuanlegen/ndern von Ansprechpartnern
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class AnsprechpartnerDialog extends AbstractDialog {
	private static final long serialVersionUID = 1L;

	/**
	 * Objekt fr ndern-Modus
	 */
	private Ansprechpartner changeingObject = null;

	/**
	 * Formular-Objekte
	 */
	private AnsprechpartnerDialog ansprechpartnerDialog = null;
	private Container contentPane = null;

	/**
	 * Eingabefelder
	 */
	private JLabel nameLabel = null;
	private JTextField nameInputField = null;
	private JLabel emailLabel = null;
	private JTextField emailInputField = null;
	private JLabel unternehmenLabel = null;
	private JComboBox unternehmenComboBox = null;
	private Object activeSelectedUnternehmen = null;

	/**
	 * Konstruktor
	 * 
	 * @param owner
	 *            Eigentmer, wichtig fr Dialog-Eigenschaft
	 * @param controller
	 *            Controller-Instanz
	 * @param label
	 *            Titel
	 * @param roleType
	 *            Benutzer mit dem sich eingeloggt wurde
	 */
	public AnsprechpartnerDialog(JFrame owner, final Controller controller,
			String label, final UserRole roleType) {
		super(owner, controller, label, roleType);
		this.ansprechpartnerDialog = this;
		this.activeDialog = this;

		JPanel ansprechpartnerPanel = new JPanel();
		ansprechpartnerPanel.setLayout(new BoxLayout(ansprechpartnerPanel,
				BoxLayout.Y_AXIS));

		/**
		 * Namensfeld
		 */
		nameLabel = new JLabel("Name:");
		nameInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(nameInputField);
		nameInputField.setDocument(new TextLengthDocument(getColumnLength(
				Ansprechpartner.class, "name")));

		/**
		 * EMailfeld
		 */
		emailLabel = new JLabel("EMail:");
		emailInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(emailInputField);
		emailInputField.setDocument(new TextLengthDocument(getColumnLength(
				Ansprechpartner.class, "email")));

		/**
		 * Unternehmensauswahlfeld
		 */
		unternehmenLabel = new JLabel("Unternehmen:");
		unternehmenComboBox = new JComboBox();
		mandatoryComponentList.add(unternehmenComboBox);
		unternehmenComboBox.addItemListener(new CustomItemListener() {
			@Override
			public void doItemChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED) {
					setActiveUnternehmen();
				}
			}
		});

		/**
		 * Objekte Panel hinzufgen
		 */
		ansprechpartnerPanel.add(nameLabel);
		ansprechpartnerPanel.add(nameInputField);
		ansprechpartnerPanel.add(unternehmenLabel);
		ansprechpartnerPanel.add(unternehmenComboBox);
		ansprechpartnerPanel.add(emailLabel);
		ansprechpartnerPanel.add(emailInputField);

		/**
		 * Vorselektion, Observer funktioniert im Konstruktor logischerweise
		 * noch nicht
		 */
		preSelection(Unternehmen.class);

		/**
		 * setzen der Panels
		 */
		setLayout(new BorderLayout());
		contentPane = getContentPane();
		contentPane.add(ansprechpartnerPanel, BorderLayout.CENTER);

		/**
		 * bernehmen des geerbten buttonPanels
		 */
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * setzen des aktuell ausgewhlten internen Unternehmens
	 */
	private void setActiveUnternehmen() {
		activeSelectedUnternehmen = null;
		if (unternehmenListe != null) {
			if (unternehmenComboBox.getItemCount() > 0) {
				if (unternehmenComboBox.getSelectedIndex() >= 0) {
					activeSelectedUnternehmen = unternehmenListe
							.get(unternehmenComboBox.getSelectedIndex());
				}
			}
		}
	}

	/**
	 * Initialisierung des Formulars, zurcksetzen der Eingabefelder
	 * 
	 * @param label
	 *            Titel
	 * @param ansprechpartner
	 *            aktueller Ansprechpartner, null bei Neuanlegen
	 */
	@Override
	public void initialize(final String label, final Object ansprechpartner) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				ansprechpartnerDialog.setTitle(label);
				ansprechpartnerDialog.changeObject = ansprechpartner;
				ansprechpartnerDialog.changeingObject = (Ansprechpartner) ansprechpartner;
				ansprechpartnerDialog.newMode = (ansprechpartner == null);

				initializeFieldsWithColors();

				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					nameInputField.setText("");
					emailInputField.setText("");
					unternehmenComboBox.setSelectedIndex(-1);
					if (unternehmenComboBox.getItemCount() > 0) {
						unternehmenComboBox.setSelectedIndex(0);
					}
				} else {
					/**
					 * ndern-Modus
					 */
					nameInputField
							.setText(ansprechpartnerDialog.changeingObject
									.getName());
					emailInputField
							.setText(ansprechpartnerDialog.changeingObject
									.getEmail());
					unternehmenComboBox
							.setSelectedItem(ansprechpartnerDialog.changeingObject
									.getUnternehmenBean().getName());
					setActiveUnternehmen();
				}
			}
		});
	};

	/**
	 * Zurckliefern der Daten ber das Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, final Object arg1) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				/**
				 * Daten in View bernehmen
				 */
				if (arg1 != null) {
					Class listType = ((ResultContent) arg1).getListType();
					List<Object> listContent = ((ResultContent) arg1)
							.getListContent();

					if (listType == Unternehmen.class) {
						updateGUIData(listType, listContent, newMode,
								unternehmenComboBox, activeSelectedUnternehmen,
								false);
					} else if (changeObject != null) {
						/**
						 * die eigene Klasse
						 */
						if (listType == changeObject.getClass()) {
							closeFormOnDeleteCascading(changeObject.getClass(),
									listContent);
						}
					}

					/**
					 * Neuzeichnen, Aufruf der validate-Methode notwendig
					 */
					ansprechpartnerDialog.validate();
					ansprechpartnerDialog.repaint();
				}
			}
		});
	}

	/**
	 * bernehmen der neuen oder genderten Daten
	 */
	@Override
	protected void saveData() {
		/**
		 * Konformitts-Abfragen
		 */
		if (nameInputField.getText().trim().equals("")) {
			errorList.add("Der Name wurde nicht eingegeben!");
			nameInputField.setBackground(defaultColorMissingMandatoryField);
		}
		if (unternehmenComboBox.getSelectedIndex() < 0) {
			errorList.add("Es wurde kein Unternehmen ausgewhlt!");
			unternehmenComboBox
					.setBackground(defaultColorMissingMandatoryField);
		}
		if (emailInputField.getText().trim().equals("")) {
			errorList.add("Es wurde keine E-Mail-Adresse eingegeben!");
			emailInputField.setBackground(defaultColorMissingMandatoryField);
		}
		checkValidEmail(emailInputField);

		/**
		 * bei Fehlern zusammengehngte Fehlermeldung ausgeben
		 */
		if (errorList.size() > 0) {
			showErrors();
		} else {
			/**
			 * Neuanlegen-Modus
			 */
			if (changeingObject == null) {
				changeingObject = new Ansprechpartner();
			}

			/**
			 * Basis-Daten bernehmen
			 */
			changeingObject.setName(nameInputField.getText().trim());
			changeingObject.setEmail(emailInputField.getText().trim());
			changeingObject
					.setUnternehmenBean(((Unternehmen) activeSelectedUnternehmen));

			/**
			 * Rmi-Methoden ausfhren
			 */
			boolean saveError = false;
			String errorString = "";
			try {
				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.insertData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Es wurde ein neuer Ansprechpartner angelegt!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				} else {
					/**
					 * ndern-Modus
					 */

					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.changeData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Der Ansprechpartner wurde gendert!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				}
			} catch (HeadlessException e) {
				saveError = true;
				errorString = e.getMessage();
			} catch (RemoteException e) {
				saveError = true;
				errorString = e.getMessage();
			}

			/**
			 * beim Auftreten eines Fehlers Fehlermeldung ausgeben
			 */
			if (saveError) {
				errorList.add("Beim Speichern ist ein Fehler aufgetreten!");
				if (!errorString.equals("")) {
					errorList.add(errorString);
				}

				showErrors();
			} else {
				closeForm();
			}
		}
	}
}