package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.rmi.RemoteException;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import main.UserRole;

import controller.ClientInterfaceImpl;
import controller.Controller;

/**
 * Login-Klasse
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
@SuppressWarnings("serial")
public class LoginFrame extends AbstractFrame {
	private JComboBox selectionComboBox = null;

	/**
	 * Konstruktor.
	 */
	public LoginFrame(final Controller controller) throws RemoteException {
		super(controller, "Login-Fenster");

		/**
		 * ComboBox mit Rollenauswahl hinzufgen, alle Werte aus Enumeration
		 * bernehmen
		 */
		JPanel selectionPanel = new JPanel();
		selectionPanel
				.setLayout(new BoxLayout(selectionPanel, BoxLayout.Y_AXIS));
		JLabel selectionLabel = new JLabel(
				"Mit welcher Rolle mchten Sie sich anmelden?");
		selectionComboBox = new JComboBox();
		for (Object sta : UserRole.values()) {
			selectionComboBox.addItem(sta.toString());
		}

		selectionPanel.add(selectionLabel);
		selectionPanel.add(selectionComboBox);

		/**
		 * Dialog zur Rollenauswahl anzeigen
		 */
		if (JOptionPane.showConfirmDialog(null, selectionPanel,
				"Rollenauswahl", JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.PLAIN_MESSAGE) == 0) {

			if (selectionComboBox.getSelectedItem().toString()
					.equals(UserRole.Student.toString())) {
				login(UserRole.Student);
			} else {
				login(UserRole.Dozent);
			}
		} else {
			System.exit(0);
		}
	}

	/**
	 * Einloggen mit Benutzerrolle
	 * 
	 * @param role
	 *            Rolle, 0=Student, 1=Dozent
	 */
	private void login(UserRole role) throws RemoteException {
		/**
		 * Anmelden als Beobachter/Observer beim Server RemoteExceptions werden
		 * nach "oben" weitergerreicht und im Controller abgehandelt
		 */
		Controller.rmiClient = new ClientInterfaceImpl();
		Controller.rmiServer.addObserver(Controller.rmiClient);

		/**
		 * nachfolgend Client-Internes anmelden
		 */
		AbstractFrame viewMF = null;

		/**
		 * View aus Frame-Liste des Controllers holen, soll nicht immer neu
		 * instanziiert werden
		 */
		viewMF = controller.getFrame(MainFrame.class);

		/**
		 * noch nicht in View-Liste des Controllers, instanziieren und
		 * hinzufgen
		 */

		if (viewMF == null) {
			viewMF = new MainFrame(controller, role);

			/**
			 * als Beobachter hinzufgen
			 */
			controller.addObserver(viewMF);

			/**
			 * der Dialog-Liste im Controller hinzufgen
			 */
			controller.addFrame(viewMF);
		}

		viewMF.setSize(800, 300);
		// viewMF.pack();
		// positionieren des Fensters: mittig
		viewMF.setLocationRelativeTo(null);
		viewMF.setVisible(true);
	}
}