package view;

import viewTables.AbstractTable;
import viewTables.AnsprechpartnerTable;
import viewTables.ProjektTable;
import viewTables.StudentTable;
import viewTables.UnternehmenTable;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import main.ResultContent;
import main.UserRole;

import controller.Controller;

import databaseModel.Ansprechpartner;
import databaseModel.Projekt;
import databaseModel.Student;
import databaseModel.Unternehmen;

/**
 * 
 * Die Hauptfenster-Klasse.
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * @version $Revision: $, $Date: $ UTC
 */
@SuppressWarnings("serial")
public class MainFrame extends AbstractFrame implements Observer {
	/* untere Buttons */
	public JButton projekteButton = null;
	public JButton ansprechpartnerButton = null;
	public JButton unternehmenButton = null;
	public JButton studentenButton = null;

	/* obere Buttons */
	private JButton newButton = null;
	private JButton changeButton = null;
	private JButton deleteButton = null;

	private JScrollPane viewScrollPane = null;
	private Container contentPane = null;
	private JLabel typeViewLabel = null;

	/**
	 * Daten-Listen
	 */
	private List<Object> projekteData = null;
	private List<Object> ansprechpartnerData = null;
	private List<Object> unternehmenData = null;
	private List<Object> studentenData = null;
	private List<Object> activeData = projekteData;

	/**
	 * JTables fr die View-Darstellung
	 */
	private ProjektTable projektTable = new ProjektTable();
	private AnsprechpartnerTable ansprechpartnerTable = new AnsprechpartnerTable();
	private UnternehmenTable unternehmenTable = new UnternehmenTable();
	private StudentTable studentTable = new StudentTable();
	private AbstractTable activeTable = projektTable;

	private final String viewDescriptionProjekte = "Projekte";
	private final String viewDescriptionAnsprechpartner = "Ansprechpartner";
	private final String viewDescriptionUnternehmen = "Unternehmen";
	private final String viewDescriptionStudenten = "Studenten";

	private final int defaultInputFieldLength = 30;

	private UserRole roleType = null;
	private MainFrame mainFrame = null;

	/**
	 * Konstruktor.
	 */
	public MainFrame(final Controller controller, final UserRole roleType) {
		super(controller, roleType.toString()
				+ " - Projektverwaltung 1.0 von (C) Thomas Kramer");
		this.roleType = roleType;
		mainFrame = this;

		// Make sure we have nice windows decorations.
		// JFrame.setDefaultLookAndFeelDecorated(true);
		addWindowListener(new securityClosing());

		/**
		 * erzeuge obere Buttonleiste
		 */
		final JPanel topButtonPanel = new JPanel();
		topButtonPanel.setLayout(new GridLayout(0, 4));
		typeViewLabel = new JLabel();
		typeViewLabel.setHorizontalAlignment(JLabel.CENTER);
		typeViewLabel.setForeground(Color.RED);
		newButton = new JButton("Neu");
		newButton.setMnemonic(KeyEvent.VK_N);
		changeButton = new JButton("ndern");
		changeButton.setMnemonic(KeyEvent.VK_E);
		deleteButton = new JButton("Lschen");
		deleteButton.setMnemonic(KeyEvent.VK_L);
		topButtonPanel.add(typeViewLabel, BorderLayout.WEST);
		topButtonPanel.add(newButton, BorderLayout.WEST);
		topButtonPanel.add(changeButton, BorderLayout.WEST);
		topButtonPanel.add(deleteButton, BorderLayout.WEST);

		/**
		 * registriere Neu-Button
		 */
		newButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (activeTable == projektTable) {
					openDialog(ProjektDialog.class, "Neues Projekt anlegen",
							true, null, null, roleType);
				} else if (activeTable == ansprechpartnerTable) {
					openDialog(AnsprechpartnerDialog.class,
							"Neuen Ansprechpartner anlegen", true, null, null,
							roleType);
				} else if (activeTable == unternehmenTable) {
					openDialog(UnternehmenDialog.class,
							"Neues Unternehmen anlegen", true, null, null,
							roleType);
				} else if (activeTable == studentTable) {
					openDialog(StudentDialog.class, "Neuen Studenten anlegen",
							true, null, null, roleType);
				}
			}
		});

		/**
		 * registriere ndern-Button
		 */
		changeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (activeTable == projektTable) {
					openDialog(ProjektDialog.class, "ndern eines Projektes",
							false, activeTable, activeData, roleType);
				} else if (activeTable == ansprechpartnerTable) {
					openDialog(AnsprechpartnerDialog.class,
							"ndern eines Ansprechpartners", false,
							activeTable, activeData, roleType);
				} else if (activeTable == unternehmenTable) {
					openDialog(UnternehmenDialog.class,
							"ndern eines Unternehmens", false, activeTable,
							activeData, roleType);
				} else if (activeTable == studentTable) {
					openDialog(StudentDialog.class, "ndern eines Studenten",
							false, activeTable, activeData, roleType);
				}
			}
		});

		/**
		 * registriere Lschen-Button
		 */
		deleteButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (activeTable.getSelectedRowCount() == 0) {
					JOptionPane
							.showMessageDialog(
									null,
									"Bitte whlen Sie einen Datensatz in der Tabelle aus.",
									"Fehler", JOptionPane.ERROR_MESSAGE);
				} else {
					int option = JOptionPane
							.showOptionDialog(
									MainFrame.this,
									"Sind Sie sicher dass Sie diesen Datensatz lschen wollen?",
									"Sicherheitsabfrage",
									JOptionPane.YES_NO_OPTION,
									JOptionPane.WARNING_MESSAGE, null, null,
									null);

					if (option == JOptionPane.YES_OPTION) {
						try {
							Controller.rmiServer.deleteData(activeData
									.get(activeTable.getSelectedRow()));
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			}
		});

		/**
		 * erzeuge untere Buttonleiste
		 */
		final JPanel bottomButtonPanel = new JPanel();
		bottomButtonPanel.setLayout(new GridLayout(0, 4));

		/**
		 * erzeuge Buttons fr untere Buttonleiste
		 */
		projekteButton = new JButton("Projekte");
		projekteButton.setMnemonic(KeyEvent.VK_P);
		ansprechpartnerButton = new JButton("Ansprechpartner");
		ansprechpartnerButton.setMnemonic(KeyEvent.VK_A);
		unternehmenButton = new JButton("Unternehmen");
		unternehmenButton.setMnemonic(KeyEvent.VK_U);
		studentenButton = new JButton("Studenten");
		studentenButton.setMnemonic(KeyEvent.VK_S);

		/**
		 * registrieren des Projekte-Buttons
		 */
		projekteButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						setActiveScrollPane(projektTable);

						try {
							Controller.rmiServer.selectData(Projekt.class);
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				});
			}
		});

		/**
		 * registrieren des Ansprechpartner-Buttons
		 */
		ansprechpartnerButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						setActiveScrollPane(ansprechpartnerTable);

						try {
							Controller.rmiServer
									.selectData(Ansprechpartner.class);
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				});
			}
		});

		/**
		 * registrieren des Unternehmen-Buttons
		 */
		unternehmenButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						setActiveScrollPane(unternehmenTable);

						try {
							Controller.rmiServer.selectData(Unternehmen.class);
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				});
			}
		});

		/**
		 * registrieren des Studenten-Buttons
		 */
		studentenButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						setActiveScrollPane(studentTable);

						try {
							Controller.rmiServer.selectData(Student.class);
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				});
			}
		});

		/**
		 * Ansichtsbuttons der unteren Buttonleiste hinzufgen
		 */
		bottomButtonPanel.add(projekteButton, BorderLayout.WEST);
		bottomButtonPanel.add(ansprechpartnerButton, BorderLayout.WEST);
		bottomButtonPanel.add(unternehmenButton, BorderLayout.WEST);
		bottomButtonPanel.add(studentenButton, BorderLayout.WEST);

		/**
		 * ContentPane managen
		 */
		setLayout(new BorderLayout());
		contentPane = getContentPane();

		/**
		 * Panels selbst hinzufgen
		 */
		contentPane.add(topButtonPanel, BorderLayout.NORTH);
		contentPane.add(bottomButtonPanel, BorderLayout.SOUTH);

		/**
		 * Vorselektion der Projekteseite
		 */
		try {
			setActiveScrollPane(projektTable);

			/**
			 * Auslieferung bers Observer-Pattern funktioniert im Konstruktor
			 * nicht!
			 */
			projekteData = Controller.rmiServer.getData(Projekt.class);
			this.update(null, new ResultContent(Projekt.class, projekteData));
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * ffnet einen Dialog im Neuanlegen- oder ndern-Modus
	 * 
	 * @param viewClass
	 *            Klasse des zu ffnenden Dialogs
	 * @param caption
	 *            Fensterbeschriftung
	 * @param newMode
	 *            Newmodus ja/nein
	 * @param activeTable
	 *            aktive JTable fr Listendarstellung
	 * @param activeData
	 *            aktive Liste von Objekten, die der aktiven JTable
	 *            zugrundeliegt
	 * @param roleType
	 *            Benutzertyp mit dem eingeloggt wurde
	 */
	private void openDialog(final Class viewClass, final String caption,
			final boolean newMode, final JTable activeTable,
			final List<Object> activeData, final UserRole roleType) {

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				boolean OK = true;
				Object changeObject = null;

				if (!newMode) {
					if (activeTable.getSelectedRow() < 0) {
						OK = false;
					} else {
						changeObject = activeData.get(activeTable
								.getSelectedRow());
					}
				}

				if (!OK) {
					JOptionPane
							.showMessageDialog(
									null,
									"Bitte whlen Sie einen Datensatz in der Tabelle aus.",
									"Fehler", JOptionPane.ERROR_MESSAGE);
				} else {
					/**
					 * View aus Dialog-Liste des Controllers holen, soll nicht
					 * immer neu instanziiert werden
					 */
					AbstractDialog view = controller.getDialog(viewClass);

					/**
					 * noch nicht in View-Liste des Controllers, instanziieren
					 * und hinzufgen
					 */
					if (view == null) {
						/**
						 * da alle Dialoge denselben Konstruktor haben kann
						 * folgender Trick angewandt werden, durch den ein
						 * expliziter Cast pro Dialog-Klasse vermieden werden
						 * kann.
						 * 
						 * Entnommen aus:
						 * http://www.coderanch.com/t/405877/java/
						 * java/Dynamic-Casting
						 */
						try {
							Constructor<?> cons = (Constructor<?>) viewClass
									.getConstructor(new Class<?>[] {
											JFrame.class, Controller.class,
											String.class, UserRole.class });
							try {
								view = (AbstractDialog) cons
										.newInstance(new Object[] { mainFrame,
												controller, caption, roleType });
							} catch (InstantiationException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (IllegalAccessException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (IllegalArgumentException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (InvocationTargetException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						} catch (NoSuchMethodException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} catch (SecurityException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						/**
						 * als Beobachter hinzufgen
						 */
						controller.addObserver(view);

						/**
						 * der Dialog-Liste im Controller hinzufgen
						 */
						controller.addDialog(view);
					}

					view.pack();
					// positionieren des Fensters: mittig
					view.setLocationRelativeTo(null);
					view.setModalityType(ModalityType.DOCUMENT_MODAL);

					/**
					 * Daten zurcksetzen und anzeigen
					 */
					view.initialize(caption, changeObject);
					view.setVisible(true);
				}
			}
		});
	}

	/**
	 * Setze aktuellen ScrollPane im ContentPane, sollte von einem Thread aus
	 * aufgerufen werden
	 * 
	 * @param jTable
	 *            JTable
	 */
	private void setActiveScrollPane(AbstractTable jTable) {
		/**
		 * JScrollPane setzen und contentPane hinzufgen
		 */
		if (viewScrollPane != null) {
			contentPane.remove(viewScrollPane);
		}
		viewScrollPane = new JScrollPane(jTable);
		contentPane.add(viewScrollPane, BorderLayout.CENTER);

		/**
		 * setzen der aktuell aktiven Daten fr die Darstellung
		 */
		activeTable = jTable;
		if (activeTable != null) {
			activeTable.clearTable();
		}

		/**
		 * setzen der Beschriftung und der Buttons
		 */
		newButton.setEnabled(true);
		changeButton.setEnabled(true);
		deleteButton.setEnabled(true);

		if (jTable == projektTable) {
			typeViewLabel.setText(viewDescriptionProjekte);

			if (roleType.equals(UserRole.Student)) {
				deleteButton.setEnabled(false);
			} else {
				newButton.setEnabled(false);
			}
		} else if (jTable == ansprechpartnerTable) {
			typeViewLabel.setText(viewDescriptionAnsprechpartner);

			if (roleType.equals(UserRole.Student)) {
				deleteButton.setEnabled(false);
			} else {
				newButton.setEnabled(false);
				changeButton.setEnabled(false);
			}
		} else if (jTable == unternehmenTable) {
			typeViewLabel.setText(viewDescriptionUnternehmen);

			if (roleType.equals(UserRole.Student)) {
				deleteButton.setEnabled(false);
			} else {
				newButton.setEnabled(false);
				changeButton.setEnabled(false);
			}
		} else if (jTable == studentTable) {
			typeViewLabel.setText(viewDescriptionStudenten);

			if (roleType.equals(UserRole.Student)) {
				newButton.setEnabled(false);
				changeButton.setEnabled(false);
				deleteButton.setEnabled(false);
			}
		}
	}

	/**
	 * Benachrichtung vom Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, final Object arg1) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				/**
				 * Daten in View bernehmen
				 */
				if (arg1 != null) {
					/**
					 * bei zurckgegebenen Daten vom Typ Projekt aktualisiere
					 * Anzeige entsprechend
					 */
					if (((ResultContent) arg1).getListType() == Projekt.class) {
						projekteData = ((ResultContent) arg1).getListContent();
						projektTable.clearTable();
						for (Object f : projekteData) {
							/**
							 * einem Projekt muss mindestens der erste Student
							 * zugeordnet werden, aber die anderen beiden sind
							 * keine Pflichtfelder - gem dem vorgegebenen
							 * Datenmodell
							 * 
							 * wenn auf Student2 und Student3 ein getName()
							 * ausgefhrt wird, wird ein INNER JOIN ausgefhrt!
							 */
							Student studentB = ((Projekt) f).getStudent2Bean();
							String studentBName = "";
							if (studentB != null) {
								studentBName = studentB.getName();
							}

							Student studentC = ((Projekt) f).getStudent3Bean();
							String studentCName = "";
							if (studentC != null) {
								studentCName = studentC.getName();
							}

							projektTable.dtm.addRow(new Object[] {
									((Projekt) f).getTitel(),
									((Projekt) f).getBeschreibung(),
									((Projekt) f).getAnsprechpartnerBean()
											.getName(),
									((Projekt) f).getStudent1Bean().getName(),
									studentBName, studentCName,
									((Projekt) f).getStatus() });
						}
					}

					/**
					 * bei zurckgegebenen Daten vom Typ Ansprechparter
					 * aktualisiere Anzeige entsprechend
					 */
					if (((ResultContent) arg1).getListType() == Ansprechpartner.class) {
						ansprechpartnerData = ((ResultContent) arg1)
								.getListContent();
						ansprechpartnerTable.clearTable();
						for (Object f : ansprechpartnerData) {
							ansprechpartnerTable.dtm.addRow(new Object[] {
									((Ansprechpartner) f).getName(),
									((Ansprechpartner) f).getEmail(),
									((Ansprechpartner) f).getUnternehmenBean()
											.getName() });
						}
					}

					/**
					 * bei zurckgegebenen Daten vom Typ Unternehmen
					 * aktualisiere Anzeige entsprechend
					 */
					if (((ResultContent) arg1).getListType() == Unternehmen.class) {
						unternehmenData = ((ResultContent) arg1)
								.getListContent();
						unternehmenTable.clearTable();
						for (Object f : unternehmenData) {
							unternehmenTable.dtm
									.addRow(new Object[] { ((Unternehmen) f)
											.getName() });
						}
					}

					/**
					 * bei zurckgegebenen Daten vom Typ Student aktualisiere
					 * Anzeige entsprechend
					 */
					if (((ResultContent) arg1).getListType() == Student.class) {
						studentenData = ((ResultContent) arg1).getListContent();
						studentTable.clearTable();
						for (Object f : studentenData) {
							studentTable.dtm.addRow(new Object[] {
									((Student) f).getName(),
									((Student) f).getMat(),
									((Student) f).getEmail() });
						}
					}

					/**
					 * setzen der aktuellen Liste von Objekten
					 */
					if (activeTable == projektTable) {
						activeData = projekteData;
					} else if (activeTable == ansprechpartnerTable) {
						activeData = ansprechpartnerData;
					} else if (activeTable == unternehmenTable) {
						activeData = unternehmenData;
					} else if (activeTable == studentTable) {
						activeData = studentenData;
					}
				}

				// Aufruf der validate-Methode fr Neuzeichnen notwendig!
				validate();
				repaint();
			}
		});
	}

	/**
	 * 
	 * private innere Klasse, um Schlieen ber X-Button abzufangen und auf
	 * jeden Fall sich beim RmiServer wieder abzumelden.
	 * 
	 * Den Shutdown-Prozess vor dem Schlieen automatisch aufzurufen (bei
	 * Benutzung des X-Buttons) funktionierte nicht wenn die MainFrame-Klasse
	 * direkt den WindowsListener implementiert - das windowClosing-Event dort
	 * wurde dann nicht aufgerufen obwohl es implementiert war.
	 * 
	 * @author $Author: Thomas Kramer
	 * @version $Revision: $, $Date: $ UTC
	 */
	private class securityClosing extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			int option = JOptionPane.showOptionDialog(MainFrame.this,
					"Wollen Sie das Programm wirklich verlassen?",
					"Sicherheitsabfrage", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE, null, null, null);

			if (option == JOptionPane.YES_OPTION) {
				try {
					Controller.rmiServer.removeObserver(Controller.rmiClient);
				} catch (RemoteException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				System.exit(0);
			} else {
				// Abbruch
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			}
		}
	}
}