package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import main.CustomItemListener;
import main.ResultContent;
import main.TextLengthDocument;
import main.UserRole;
import main.ProjektStatus;
import controller.Controller;
import databaseModel.Ansprechpartner;
import databaseModel.Projekt;
import databaseModel.Student;

/**
 * Klasse zum Neuanlegen/ndern eines Projektes
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ProjektDialog extends AbstractDialog {
	private static final long serialVersionUID = 1L;

	/**
	 * Objekt fr ndern-Modus
	 */
	private Projekt changeingObject = null;

	/**
	 * Formular-Objekte
	 */
	private ProjektDialog projektDialog = null;
	private Container contentPane = null;

	/**
	 * Eingabefelder
	 */
	private JLabel titleLabel = null;
	private JTextField titleInputField = null;
	private JLabel beschreibungLabel = null;
	private JTextField beschreibungInputField = null;

	private JLabel ansprechpartnerLabel = null;
	private JComboBox ansprechpartnerComboBox = null;
	private Object activeSelectedAnsprechpartner = null;

	private JLabel student1Label = null;
	private JComboBox student1ComboBox = null;
	private Object activeSelectedStudent1 = null;

	private JLabel student2Label = null;
	private JComboBox student2ComboBox = null;
	private Object activeSelectedStudent2 = null;

	private JLabel student3Label = null;
	private JComboBox student3ComboBox = null;
	private Object activeSelectedStudent3 = null;

	private JLabel statusLabel = null;
	private JComboBox statusComboBox = null;
	private JLabel kommentarLabel = null;
	private JTextArea kommentarTextArea = null;

	/**
	 * Vorgabelnge fr Eingabefelder
	 */
	private int defaultInputFieldLength = 30;

	/**
	 * Konstruktor
	 * 
	 * @param owner
	 *            Eigentmer, wichtig fr Dialog-Eigenschaft
	 * @param controller
	 *            Controller-Instanz
	 * @param label
	 *            Titel
	 * @param roleType
	 *            Benutzer mit dem sich eingeloggt wurde
	 */
	public ProjektDialog(JFrame owner, final Controller controller,
			String label, final UserRole roleType) {
		super(owner, controller, label, roleType);
		this.projektDialog = this;
		this.activeDialog = this;

		JPanel projektPanel = new JPanel();
		projektPanel.setLayout(new BoxLayout(projektPanel, BoxLayout.Y_AXIS));

		/**
		 * Titelfeld
		 */
		titleLabel = new JLabel("Titel:");
		titleInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(titleInputField);
		titleInputField.setDocument(new TextLengthDocument(getColumnLength(
				Projekt.class, "titel")));

		/**
		 * Beschreibungsfeld
		 */
		beschreibungLabel = new JLabel("Beschreibung:");
		beschreibungInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(beschreibungInputField);
		beschreibungInputField.setDocument(new TextLengthDocument(
				getColumnLength(Projekt.class, "beschreibung")));

		/**
		 * Ansprechpartnerauswahlfeld
		 */
		ansprechpartnerLabel = new JLabel("Ansprechpartner:");
		ansprechpartnerComboBox = new JComboBox();
		mandatoryComponentList.add(ansprechpartnerComboBox);
		ansprechpartnerComboBox.addItemListener(new CustomItemListener() {
			@Override
			public void doItemChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED) {
					setActiveAnsprechpartner();
				}
			}
		});

		/**
		 * Studentenauswahlfelder
		 */
		student1Label = new JLabel("Student1:");
		student1ComboBox = new JComboBox();
		mandatoryComponentList.add(student1ComboBox);
		student1ComboBox.addItemListener(new CustomItemListener() {
			@Override
			public void doItemChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED) {
					setActiveStudent1();
				}
			}
		});

		student2Label = new JLabel("Student2:");
		student2ComboBox = new JComboBox();
		mandatoryComponentList.add(student2ComboBox);
		student2ComboBox.addItemListener(new CustomItemListener() {
			@Override
			public void doItemChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED) {
					setActiveStudent2();
				}
			}
		});

		student3Label = new JLabel("Student3:");
		student3ComboBox = new JComboBox();
		mandatoryComponentList.add(student3ComboBox);
		student3ComboBox.addItemListener(new CustomItemListener() {
			@Override
			public void doItemChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED) {
					setActiveStudent3();
				}
			}
		});

		/**
		 * Statusauswahlfeld
		 */
		statusLabel = new JLabel("Status:");
		statusComboBox = new JComboBox();
		mandatoryComponentList.add(statusComboBox);
		for (Object sta : ProjektStatus.values()) {
			statusComboBox.addItem(sta.toString());
		}

		/**
		 * Kommentarfeld
		 */
		kommentarLabel = new JLabel("Kommentar:");
		kommentarTextArea = new JTextArea(defaultTextAreaRows,
				defaultTextAreaColumns);
		kommentarTextArea.setDocument(new TextLengthDocument(getColumnLength(
				Projekt.class, "kommentar")));

		/**
		 * Rollenunterscheidung
		 */
		titleInputField.setEnabled(roleType.equals(UserRole.Student));
		beschreibungInputField.setEnabled(roleType.equals(UserRole.Student));
		ansprechpartnerComboBox.setEnabled(roleType.equals(UserRole.Student));
		student1ComboBox.setEnabled(roleType.equals(UserRole.Student));
		student2ComboBox.setEnabled(roleType.equals(UserRole.Student));
		student3ComboBox.setEnabled(roleType.equals(UserRole.Student));
		statusComboBox.setEnabled(!roleType.equals(UserRole.Student));
		kommentarTextArea.setEditable(!roleType.equals(UserRole.Student));

		/**
		 * Objekte Panel hinzufgen
		 */
		projektPanel.add(titleLabel);
		projektPanel.add(titleInputField);
		projektPanel.add(beschreibungLabel);
		projektPanel.add(beschreibungInputField);
		projektPanel.add(ansprechpartnerLabel);
		projektPanel.add(ansprechpartnerComboBox);
		projektPanel.add(student1Label);
		projektPanel.add(student1ComboBox);
		projektPanel.add(student2Label);
		projektPanel.add(student2ComboBox);
		projektPanel.add(student3Label);
		projektPanel.add(student3ComboBox);
		projektPanel.add(statusLabel);
		projektPanel.add(statusComboBox);
		projektPanel.add(kommentarLabel);
		projektPanel.add(kommentarTextArea);

		/**
		 * Vorselektion, Observer funktioniert im Konstruktor logischerweise
		 * noch nicht
		 */
		preSelection(Ansprechpartner.class);
		preSelection(Student.class);

		/**
		 * setzen der Panels
		 */
		setLayout(new BorderLayout());
		contentPane = getContentPane();
		contentPane.add(projektPanel, BorderLayout.CENTER);

		/**
		 * bernehmen des geerbten buttonPanels
		 */
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * setzen des aktuell ausgewhlten internen Ansprechpartners
	 */
	private void setActiveAnsprechpartner() {
		activeSelectedAnsprechpartner = null;
		if (ansprechpartnerListe != null) {
			if (ansprechpartnerComboBox.getItemCount() > 0) {
				if (ansprechpartnerComboBox.getSelectedIndex() >= 0) {
					activeSelectedAnsprechpartner = ansprechpartnerListe
							.get(ansprechpartnerComboBox.getSelectedIndex());
				}
			}
		}
	}

	/**
	 * setzen des aktuell ausgewhlten internen 1. Studenten
	 */
	private void setActiveStudent1() {
		activeSelectedStudent1 = null;
		if (studentenListe != null) {
			if (student1ComboBox.getItemCount() > 0) {
				if (student1ComboBox.getSelectedIndex() >= 0) {
					activeSelectedStudent1 = studentenListe
							.get(student1ComboBox.getSelectedIndex());
				}
			}
		}
	}

	/**
	 * setzen des aktuell ausgewhlten internen 2. Studenten
	 */
	private void setActiveStudent2() {
		activeSelectedStudent2 = null;
		if (studentenListe != null) {
			if (student2ComboBox.getItemCount() > 0) {
				if (student2ComboBox.getSelectedIndex() > 0) {
					activeSelectedStudent2 = studentenListe
							.get(student2ComboBox.getSelectedIndex() - 1);
				}
			}
		}
	}

	/**
	 * setzen des aktuell ausgewhlten internen 3. Studenten
	 */
	private void setActiveStudent3() {
		activeSelectedStudent3 = null;
		if (studentenListe != null) {
			if (student3ComboBox.getItemCount() > 0) {
				if (student3ComboBox.getSelectedIndex() > 0) {
					activeSelectedStudent3 = studentenListe
							.get(student3ComboBox.getSelectedIndex() - 1);
				}
			}
		}
	}

	/**
	 * Initialisierung des Formulars, zurcksetzen der Eingabefelder
	 * 
	 * @param label
	 *            Titel *
	 * @param projekt
	 *            aktuelles Projekt, null bei Neuanlegen
	 */
	@Override
	public void initialize(final String label, final Object projekt) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				projektDialog.setTitle(label);
				projektDialog.changeObject = projekt;
				projektDialog.changeingObject = (Projekt) projekt;
				projektDialog.newMode = (projekt == null);

				initializeFieldsWithColors();

				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					titleInputField.setText("");
					beschreibungInputField.setText("");
					ansprechpartnerComboBox.setSelectedIndex(-1);
					if (ansprechpartnerComboBox.getItemCount() > 0) {
						ansprechpartnerComboBox.setSelectedIndex(0);
					}
					student1ComboBox.setSelectedIndex(-1);
					if (student1ComboBox.getItemCount() > 0) {
						student1ComboBox.setSelectedIndex(0);
					}
					student2ComboBox.setSelectedIndex(-1);
					student3ComboBox.setSelectedIndex(-1);
					statusComboBox.setSelectedItem(ProjektStatus.offen
							.toString());
					kommentarTextArea.setText("");
				} else {
					/**
					 * ndern-Modus
					 */
					titleInputField.setText(projektDialog.changeingObject
							.getTitel());
					beschreibungInputField
							.setText(projektDialog.changeingObject
									.getBeschreibung());

					ansprechpartnerComboBox
							.setSelectedItem(projektDialog.changeingObject
									.getAnsprechpartnerBean().getName());
					setActiveAnsprechpartner();
					student1ComboBox
							.setSelectedItem(projektDialog.changeingObject
									.getStudent1Bean().getName());
					setActiveStudent1();

					/**
					 * INNER JOIN vermeiden, Student2 und Student3 sind keine
					 * Pflichtfelder!
					 */

					Student studentB = projektDialog.changeingObject
							.getStudent2Bean();
					student2ComboBox.setSelectedIndex(-1);
					if (studentB != null) {
						student2ComboBox.setSelectedItem(studentB.getName());
					}
					setActiveStudent2();

					Student studentC = projektDialog.changeingObject
							.getStudent3Bean();
					student3ComboBox.setSelectedIndex(-1);
					if (studentC != null) {
						student3ComboBox.setSelectedItem(studentC.getName());
					}
					setActiveStudent3();

					String status = projektDialog.changeingObject.getStatus();
					statusComboBox.setSelectedIndex(-1);
					if (!status.equals("")) {
						statusComboBox.setSelectedItem(status);
					}

					/**
					 * der Projektstatus darf nicht mehr nachtrglich gendert
					 * werden knnen, wenn er bereits einmal auf angenommen oder
					 * abgelehnt gesetzt wurde
					 * 
					 * erweiterte Freischaltungsregel, vergleiche mit Regel im
					 * Konstruktor
					 */
					statusComboBox.setEnabled(!roleType
							.equals(UserRole.Student)
							&& projektDialog.changeingObject.getStatus()
									.equals(ProjektStatus.offen.toString()));

					kommentarTextArea.setText(projektDialog.changeingObject
							.getKommentar());
				}
			}
		});
	}

	/**
	 * Zurckliefern der Daten ber das Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, final Object arg1) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				/**
				 * Daten in View bernehmen
				 */
				if (arg1 != null) {
					Class listType = ((ResultContent) arg1).getListType();
					List<Object> listContent = ((ResultContent) arg1)
							.getListContent();

					if (listType == Ansprechpartner.class) {
						updateGUIData(listType, listContent, newMode,
								ansprechpartnerComboBox,
								activeSelectedAnsprechpartner, false);
					} else if (listType == Student.class) {
						updateGUIData(listType, listContent, newMode,
								student1ComboBox, activeSelectedStudent1, false);

						updateGUIData(listType, listContent, newMode,
								student2ComboBox, activeSelectedStudent2, true);

						updateGUIData(listType, listContent, newMode,
								student3ComboBox, activeSelectedStudent3, true);
					} else if (changeObject != null) {
						/**
						 * die eigene Klasse
						 */
						if (listType == changeObject.getClass()) {
							closeFormOnDeleteCascading(changeObject.getClass(),
									listContent);
						}
					}

					/**
					 * Neuzeichnen, Aufruf der validate-Methode notwendig
					 */
					projektDialog.validate();
					projektDialog.repaint();
				}
			}
		});
	}

	/**
	 * bernehmen der neuen oder genderten Daten
	 */
	@Override
	protected void saveData() {
		/**
		 * berprfen ob Datensatz bereits vorhanden ist
		 */
		checkExistingRecord(Projekt.class, "titel", titleInputField);

		/**
		 * Konformitts-Abfragen
		 */
		if (titleInputField.getText().trim().equals("")) {
			errorList.add("Das Titel-Feld wurde nicht ausgefllt!");
			titleInputField.setBackground(defaultColorMissingMandatoryField);
		}
		if (beschreibungInputField.getText().trim().equals("")) {
			errorList.add("Das Beschreibungs-Feld wurde nicht ausgefllt!");
			beschreibungInputField
					.setBackground(defaultColorMissingMandatoryField);
		}
		if (ansprechpartnerComboBox.getSelectedIndex() < 0) {
			errorList.add("Es wurde kein Ansprechpartner ausgewhlt!");
			ansprechpartnerComboBox
					.setBackground(defaultColorMissingMandatoryField);
		}
		if (student1ComboBox.getSelectedIndex() < 0) {
			errorList.add("Bitte whlen Sie den ersten Studenten aus!");
			student1ComboBox.setBackground(defaultColorMissingMandatoryField);
		}
		if (statusComboBox.getSelectedIndex() < 0) {
			errorList.add("Bitte geben Sie den Status an!");
			statusComboBox.setBackground(defaultColorMissingMandatoryField);
		}

		/**
		 * Studenten drfen nicht mehrmals ausgewhlt werden knnen
		 */
		String student1 = student1ComboBox.getSelectedItem().toString();
		String student2 = "";
		if (student2ComboBox.getSelectedItem() != null) {
			student2 = student2ComboBox.getSelectedItem().toString();
		}
		String student3 = "";
		if (student3ComboBox.getSelectedItem() != null) {
			student3 = student3ComboBox.getSelectedItem().toString();
		}

		if (student2.equals(student1) || student3.equals(student1)
				|| (student3.equals(student2) && !student3.equals(""))) {
			errorList
					.add("Es drfen keine Studenten mehrmals ausgewhlt werden!");
			student1ComboBox.setBackground(defaultColorMissingMandatoryField);
			student2ComboBox.setBackground(defaultColorMissingMandatoryField);
			student3ComboBox.setBackground(defaultColorMissingMandatoryField);
		}

		/**
		 * bei Fehlern zusammengehngte Fehlermeldung ausgeben
		 */
		if (errorList.size() > 0) {
			showErrors();
		} else {
			/**
			 * Neuanlegen-Modus
			 */
			if (changeingObject == null) {
				changeingObject = new Projekt();
			}

			/**
			 * Basis-Daten bernehmen
			 */
			changeingObject.setTitel(titleInputField.getText().trim());
			changeingObject.setBeschreibung(beschreibungInputField.getText()
					.trim());
			changeingObject
					.setAnsprechpartnerBean((Ansprechpartner) activeSelectedAnsprechpartner);
			changeingObject.setStudent1Bean((Student) activeSelectedStudent1);
			changeingObject.setStudent2Bean((Student) activeSelectedStudent2);
			changeingObject.setStudent3Bean((Student) activeSelectedStudent3);

			/**
			 * Rmi-Methoden ausfhren
			 */
			boolean saveError = false;
			String errorString = "";
			try {
				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					/**
					 * Vorbelegung der Restdaten
					 */
					changeingObject.setStatus(ProjektStatus.offen.toString());
					changeingObject.setKommentar("");

					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.insertData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Es wurde ein neues Projekt angelegt!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				} else {
					/**
					 * ndern-Modus
					 */

					/**
					 * Restdaten bernehmen
					 */
					if (statusComboBox.getSelectedItem() != null) {
						changeingObject.setStatus(statusComboBox
								.getSelectedItem().toString());
					} else {
						changeingObject.setStatus(ProjektStatus.offen
								.toString());
					}
					changeingObject.setKommentar(kommentarTextArea.getText());

					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.changeData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Das Projekt wurde gendert!", "Information",
								JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				}
			} catch (HeadlessException e) {
				saveError = true;
				errorString = e.getMessage();
			} catch (RemoteException e) {
				saveError = true;
				errorString = e.getMessage();
			}

			/**
			 * beim Auftreten eines Fehlers Fehlermeldung ausgeben
			 */
			if (saveError) {
				errorList.add("Beim Speichern ist ein Fehler aufgetreten!");
				if (!errorString.equals("")) {
					errorList.add(errorString);
				}

				showErrors();
			} else {
				closeForm();
			}
		}
	}
}
