package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import main.ResultContent;
import main.TextLengthDocument;
import main.UserRole;
import controller.Controller;
import databaseModel.Ansprechpartner;
import databaseModel.Student;
import databaseModel.Unternehmen;

/**
 * Klasse zum Neuanlegen/ndern von Studenten
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class StudentDialog extends AbstractDialog {
	/**
	 * Objekt fr ndern-Modus
	 */
	private Student changeingObject = null;

	/**
	 * Formular-Objekte
	 */
	private StudentDialog studentDialog = null;
	private Container contentPane = null;

	/**
	 * Eingabefelder
	 */
	private JLabel nameLabel = null;
	private JTextField nameInputField = null;
	private JLabel matLabel = null;
	private JTextField matInputField = null;
	private JLabel emailLabel = null;
	private JTextField emailInputField = null;

	/**
	 * Konstruktor
	 * 
	 * @param owner
	 *            Eigentmer, wichtig fr Dialog-Eigenschaft
	 * @param controller
	 *            Controller-Instanz
	 * @param label
	 *            Titel
	 * @param roleType
	 *            Benutzer mit dem sich eingeloggt wurde
	 */
	public StudentDialog(JFrame owner, Controller controller, String label,
			UserRole roleType) {
		super(owner, controller, label, roleType);
		this.studentDialog = this;
		this.activeDialog = this;

		JPanel studentPanel = new JPanel();
		studentPanel.setLayout(new BoxLayout(studentPanel, BoxLayout.Y_AXIS));

		/**
		 * Namenfeld
		 */
		nameLabel = new JLabel("Name:");
		nameInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(nameInputField);
		nameInputField.setDocument(new TextLengthDocument(getColumnLength(
				Student.class, "name")));

		/**
		 * MAT-Feld
		 */
		matLabel = new JLabel("Matrikelnummer:");
		matInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(matInputField);
		matInputField.setDocument(new TextLengthDocument(getColumnLength(
				Student.class, "mat")));

		/**
		 * EMail-Feld
		 */
		emailLabel = new JLabel("EMail:");
		emailInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(emailInputField);
		emailInputField.setDocument(new TextLengthDocument(getColumnLength(
				Student.class, "email")));

		/**
		 * Objekte Panel hinzufgen
		 */
		studentPanel.add(nameLabel);
		studentPanel.add(nameInputField);
		studentPanel.add(matLabel);
		studentPanel.add(matInputField);
		studentPanel.add(emailLabel);
		studentPanel.add(emailInputField);

		/**
		 * setzen der Panels
		 */
		setLayout(new BorderLayout());
		contentPane = getContentPane();
		contentPane.add(studentPanel, BorderLayout.CENTER);

		/**
		 * bernehmen des geerbten buttonPanels
		 */
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Initialisierung des Formulars, zurcksetzen der Eingabefelder
	 * 
	 * @param label
	 *            Titel
	 * @param student
	 *            aktueller Student, null bei Neuanlegen
	 */
	@Override
	public void initialize(final String label, final Object student) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				studentDialog.setTitle(label);
				studentDialog.changeObject = student;
				studentDialog.changeingObject = (Student) student;
				studentDialog.newMode = (student == null);

				initializeFieldsWithColors();

				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					nameInputField.setText("");
					matInputField.setText("");
					emailInputField.setText("");
				} else {
					/**
					 * ndern-Modus
					 */
					nameInputField.setText(studentDialog.changeingObject
							.getName());
					matInputField.setText(Integer
							.toString(studentDialog.changeingObject.getMat()));
					emailInputField.setText(studentDialog.changeingObject
							.getEmail());
				}
			}
		});
	};

	/**
	 * bernehmen der neuen oder genderten Daten
	 */
	@Override
	protected void saveData() {
		/**
		 * Konformitts-Abfragen
		 */
		if (nameInputField.getText().trim().equals("")) {
			errorList.add("Der Name wurde nicht eingegeben!");
			nameInputField.setBackground(defaultColorMissingMandatoryField);
		}
		if (matInputField.getText().trim().equals("")) {
			errorList.add("Es wurde keine Matrikelnummer eingegeben!");
			matInputField.setBackground(defaultColorMissingMandatoryField);
		}
		checkValidOnlyNumbers(matInputField);
		if (emailInputField.getText().trim().equals("")) {
			errorList.add("Es wurde keine E-Mail-Adresse eingegeben!");
			emailInputField.setBackground(defaultColorMissingMandatoryField);
		}
		checkValidEmail(emailInputField);

		/**
		 * bei Fehlern zusammengehngte Fehlermeldung ausgeben
		 */
		if (errorList.size() > 0) {
			showErrors();
		} else {
			/**
			 * Neuanlegen-Modus
			 */
			if (changeingObject == null) {
				changeingObject = new Student();
			}

			/**
			 * Basis-Daten bernehmen
			 */
			changeingObject.setName(nameInputField.getText().trim());
			changeingObject.setMat(Integer.parseInt(matInputField.getText()
					.trim()));
			changeingObject.setEmail(emailInputField.getText().trim());

			/**
			 * Rmi-Methoden ausfhren
			 */
			boolean saveError = false;
			String errorString = "";
			try {
				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.insertData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Es wurde ein neuer Student angelegt!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				} else {
					/**
					 * ndern-Modus
					 */

					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.changeData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Der Student wurde gendert!", "Information",
								JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				}
			} catch (HeadlessException e) {
				saveError = true;
				errorString = e.getMessage();
			} catch (RemoteException e) {
				saveError = true;
				errorString = e.getMessage();
			}

			/**
			 * beim Auftreten eines Fehlers Fehlermeldung ausgeben
			 */
			if (saveError) {
				errorList.add("Beim Speichern ist ein Fehler aufgetreten!");
				if (!errorString.equals("")) {
					errorList.add(errorString);
				}

				showErrors();
			} else {
				closeForm();
			}
		}
	}
}