package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import main.ResultContent;
import main.TextLengthDocument;
import main.UserRole;
import controller.Controller;
import databaseModel.Ansprechpartner;
import databaseModel.Unternehmen;

/**
 * Klasse zum Neuanlegen/ndern von Unternehmen
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class UnternehmenDialog extends AbstractDialog {
	/**
	 * Objekt fr ndern-Modus
	 */
	private Unternehmen changeingObject = null;

	/**
	 * Formular-Objekte
	 */
	private UnternehmenDialog unternehmenDialog = null;
	private Container contentPane = null;

	/**
	 * Eingabefelder
	 */
	private JLabel nameLabel = null;
	private JTextField nameInputField = null;

	/**
	 * Konstruktor
	 * 
	 * @param owner
	 *            Eigentmer, wichtig fr Dialog-Eigenschaft
	 * @param controller
	 *            Controller-Instanz
	 * @param label
	 *            Titel
	 * @param roleType
	 *            Benutzer mit dem sich eingeloggt wurde
	 */
	public UnternehmenDialog(JFrame owner, final Controller controller,
			String label, final UserRole roleType) {
		super(owner, controller, label, roleType);
		this.unternehmenDialog = this;
		this.activeDialog = this;

		JPanel unternehmenPanel = new JPanel();
		unternehmenPanel.setLayout(new BoxLayout(unternehmenPanel,
				BoxLayout.Y_AXIS));

		/**
		 * Namenfeld
		 */
		nameLabel = new JLabel("Bitte geben Sie den Namen ein:");
		nameInputField = new JTextField(defaultVisibleFieldLength);
		mandatoryComponentList.add(nameInputField);
		nameInputField.setDocument(new TextLengthDocument(getColumnLength(
				Unternehmen.class, "name")));

		/**
		 * Objekte Panel hinzufgen
		 */
		unternehmenPanel.add(nameLabel);
		unternehmenPanel.add(nameInputField);

		/**
		 * setzen der Panels
		 */
		setLayout(new BorderLayout());
		contentPane = getContentPane();
		contentPane.add(unternehmenPanel, BorderLayout.CENTER);

		/**
		 * bernehmen des geerbten buttonPanels
		 */
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Initialisierung des Formulars, zurcksetzen der Eingabefelder
	 * 
	 * @param label
	 *            Titel
	 * @param unternehmen
	 *            aktuelles Unternehmen, null bei Neuanlegen
	 */
	@Override
	public void initialize(final String label, final Object unternehmen) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				unternehmenDialog.setTitle(label);
				unternehmenDialog.changeObject = unternehmen;
				unternehmenDialog.changeingObject = (Unternehmen) unternehmen;
				unternehmenDialog.newMode = (unternehmen == null);

				initializeFieldsWithColors();

				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					nameInputField.setText("");
				} else {
					/**
					 * ndern-Modus
					 */
					nameInputField.setText(unternehmenDialog.changeingObject
							.getName());
				}
			}
		});
	};

	/**
	 * bernehmen der neuen oder genderten Daten
	 */
	@Override
	protected void saveData() {
		/**
		 * berprfen ob Datensatz bereits vorhanden ist
		 */
		checkExistingRecord(Unternehmen.class, "name", nameInputField);

		/**
		 * Konformitts-Abfragen
		 */
		if (nameInputField.getText().trim().equals("")) {
			errorList.add("Der Name wurde nicht eingegeben!");
			nameInputField.setBackground(defaultColorMissingMandatoryField);
		}

		/**
		 * bei Fehlern zusammengehngte Fehlermeldung ausgeben
		 */
		if (errorList.size() > 0) {
			showErrors();
		} else {
			/**
			 * Neuanlegen-Modus
			 */
			if (changeingObject == null) {
				changeingObject = new Unternehmen();
			}

			/**
			 * Basis-Daten bernehmen
			 */
			changeingObject.setName(nameInputField.getText().trim());

			/**
			 * Rmi-Methoden ausfhren
			 */
			boolean saveError = false;
			String errorString = "";
			try {
				/**
				 * Neuanlegen-Modus
				 */
				if (newMode) {
					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.insertData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Es wurde ein neues Unternehmen angelegt!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				} else {
					/**
					 * ndern-Modus
					 */

					/**
					 * Daten speichern
					 */
					if (Controller.rmiServer.changeData(changeingObject)) {
						JOptionPane.showMessageDialog(null,
								"Das Unternehmen wurde gendert!",
								"Information", JOptionPane.INFORMATION_MESSAGE);

					} else {
						/**
						 * Fehlermeldungen bei der Server-Verarbeitung werden in
						 * der Log-Datei des Servers geloggt!
						 */
						saveError = true;
					}
				}
			} catch (HeadlessException e) {
				saveError = true;
				errorString = e.getMessage();
			} catch (RemoteException e) {
				saveError = true;
				errorString = e.getMessage();
			}

			/**
			 * beim Auftreten eines Fehlers Fehlermeldung ausgeben
			 */
			if (saveError) {
				errorList.add("Beim Speichern ist ein Fehler aufgetreten!");
				if (!errorString.equals("")) {
					errorList.add(errorString);
				}

				showErrors();
			} else {
				closeForm();
			}
		}
	}
}